@@ -1,5 +1,5 @@
 #
-#   Copyright 2013 The OpenMx Project
+#   Copyright 2013-2018 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -278,10 +278,9 @@ setClass(Class = "MxComputeGradientDescent",
 	     availableEngines = "character",
 	     tolerance = "numeric",
 	   nudgeZeroStarts = "MxCharOrLogical",
-	   .excludeVars = "MxOptionalChar",
 	   verbose = "integer",
 	     maxMajorIter = "integer",
-	     gradientAlgo = "character",
+	     gradientAlgo = "MxOptionalChar",
 	     gradientIterations = "integer",
 	   gradientStepSize = "numeric",
 	   defaultCImethod = "character",
@@ -328,7 +327,6 @@ setMethod("initialize", "MxComputeGradientDescent",
 		  if (imxHasNPSOL()) {
 			  .Object@availableEngines <- c(.Object@availableEngines, "NPSOL")
 		  }
-		  .Object@.excludeVars <- c()
 		  .Object
 	  })
 
@@ -343,24 +341,17 @@ imxHasNPSOL <- function() .Call(hasNPSOL_wrapper)
 ##'
 ##' This optimizer does not require analytic derivatives of the fit
 ##' function. The fully open-source CRAN version of OpenMx offers 2 choices,
-##' SLSQP (from the NLOPT collection) and CSOLNP.  The OpenMx Team's version of
-##' OpenMx offers the choice of three optimizers: SLSQP, CSOLNP, and NPSOL.
+##' CSOLNP and SLSQP (from the NLOPT collection).  The OpenMx Team's version of
+##' OpenMx offers the choice of three optimizers: CSOLNP, SLSQP, and NPSOL.
 ##'
-##' One of the most important options for SLSQP is
-##' \code{gradientAlgo}. By default, the \code{central} method 
-##' is used.  This method requires 2 times 
+##' One option for CSOLNP and SLSQP is
+##' \code{gradientAlgo}. CSOLNP uses \code{forward} method
+##' by default, while SLSQP uses \code{central} method. \code{forward} method requires
+##' 1 time \code{gradientIterations} function evaluation per parameter
+##' per gradient, while \code{central} method requires 2 times
 ##' \code{gradientIterations} function evaluations per parameter 
-##' per gradient.  The \code{central} method can be much more accurate 
-##' than the \code{forward} method, which requires 
-##' \code{gradientIterations} function evaluations per parameter per 
-##' gradient.  The \code{forward} method is faster, and often works
-##' well enough, but can result in imprecise gradient estimations that
-##' may not allow SLSQP to fully optimize a given model, possibly 
-##' resulting in code RED (status code 5 or 6).
-##' 
-##' Currently, only SLSQP uses arguments \code{gradientIterations} 
-##' and \code{gradientAlgo}.  CSOLNP always uses the \code{forward} 
-##' method; NPSOL usually uses the \code{forward} method, but 
+##' per gradient. Users can change the default methods for either of these optimizers.
+##' NPSOL usually uses the \code{forward} method, but
 ##' adaptively switches to \code{central} under certain circumstances.
 ##' 
 ##' CSOLNP uses the value of argument \code{gradientStepSize} as-is, 
@@ -371,12 +362,12 @@ imxHasNPSOL <- function() .Call(hasNPSOL_wrapper)
 ##' of \link{mxOption} \dQuote{Function precision} to determine its gradient
 ##' step size.
 ##' 
-##' Currently, only SLSQP and NPSOL can use analytic gradients, 
+##' All three optimizers can use analytic gradients,
 ##' and only NPSOL uses \code{warmStart}.
 ##'
 ##' @param freeSet names of matrices containing free parameters.
 ##' @param ...  Not used.  Forces remaining arguments to be specified by name.
-##' @param engine specific 'NPSOL', 'SLSQP', or 'CSOLNP'
+##' @param engine specific 'CSOLNP', 'SLSQP', or 'NPSOL'
 ##' @param fitfunction name of the fitfunction (defaults to 'fitfunction')
 ##' @param verbose level of debugging output
 ##' @param tolerance how close to the optimum is close enough (also known as the optimality tolerance)
@@ -664,9 +655,8 @@ setMethod("initialize", "MxComputeConfidenceInterval",
 ##' There are various equivalent ways to pose the optimization
 ##' problems required to estimate confindence intervals. Most accurate
 ##' solutions are achieved when the problem is posed using non-linear
-##' constraints. However, the available optimizers (NPSOL, SLSQP, and
-##' CSOLNP) often have difficulty with non-linear
-##' constraints.
+##' constraints. However, the available optimizers (CSOLNP, SLSQP, and NPSOL) often have difficulty with non-linear
+##' constraints. 
 ##' 
 ##' @param plan compute plan to optimize the model
 ##' @param ...  Not used.  Forces remaining arguments to be specified by name.
@@ -1311,7 +1301,6 @@ setClass(
 		nudgeZeroStarts="MxCharOrLogical",
 		maxIter="integer",
 		defaultMaxIter="logical",
-		.excludeVars="MxOptionalChar",
 		alpha="numeric",
 		betao="numeric",
 		betai="numeric",
@@ -1354,7 +1343,6 @@ setMethod(
 		.Object@nudgeZeroStarts <- nudgeZeroStarts
 		.Object@defaultMaxIter <- ifelse(length(maxIter),FALSE,TRUE)
 		.Object@maxIter <- as.integer(maxIter)
-		.Object@.excludeVars <- c()
 		
 		.Object@alpha <- alpha
 		.Object@betao <- betao
