@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2017 The OpenMx Project
+#   Copyright 2007-2018 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -45,8 +45,6 @@ setClass(Class = "MxExpectationLISREL",
 		numStats = "numeric",
 		thresholds = "MxCharOrNumber",
 		dims = "character",
-		thresholdColumns = "numeric", #Used in FIML
-		thresholdLevels = "numeric", # Used in FIML
 		threshnames = "character",
 		depth = "integer"), #Used to speed up I-A inverse in RAM, could be used to speed up I-B inverse in LISREL
 	contains = "BaseExpectationNormal")
@@ -460,9 +458,6 @@ setMethod("genericExpFunConvert", signature("MxExpectationLISREL"),
 			.Object@dataColumns <- generateDataColumns(flatModel, translatedNames, data)
 			verifyThresholds(flatModel, model, labelsData, data, translatedNames, threshName)
 			.Object@thresholds <- imxLocateIndex(flatModel, threshName, name)
-			retval <- generateThresholdColumns(flatModel, model, labelsData, translatedNames, data, threshName)
-			.Object@thresholdColumns <- retval[[1]]
-			.Object@thresholdLevels <- retval[[2]]
 			if (length(mxDataObject@observed) == 0) {
 				.Object@data <- as.integer(NA)
 			}
@@ -702,7 +697,7 @@ setMethod("genericGetExpected", signature("MxExpectationLISREL"),
 		  }
 		  if ('means' %in% what) {
 			  if(single.na(TXname) & single.na(TYname)){
-					warning("Means requested, but model has no means.\nAdd appropriate TX, TY, KA, and/or KA matrices to get real means.")
+					warning("Means requested, but model has no means.\nAdd appropriate TX, TY, KA, and/or AL matrices to get real means.")
 				  mean <- matrix( , 0, 0)
 			  } else {
 					if(hasX & (single.na(TXname) || single.na(KAname)) ){
