@@ -1,5 +1,5 @@
  /*
- *  Copyright 2007-2017 The OpenMx Project
+ *  Copyright 2007-2018 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -228,13 +228,17 @@ void omxGREMLExpectation::populateAttr(SEXP algebra) {
 
   omxGREMLExpectation* oge = this;
   
-  Rf_setAttrib(algebra, Rf_install("numStats"), Rf_ScalarReal(oge->y->dataMat->cols));
-  Rf_setAttrib(algebra, Rf_install("numFixEff"), Rf_ScalarInteger(oge->X->cols));
+  {
+  ProtectedSEXP RnumStat(Rf_ScalarReal(oge->y->dataMat->cols));
+  Rf_setAttrib(algebra, Rf_install("numStats"), RnumStat);
+  ProtectedSEXP RnumFixEff(Rf_ScalarInteger(oge->X->cols));
+  Rf_setAttrib(algebra, Rf_install("numFixEff"), RnumFixEff);
+  }
   
   Eigen::Map< Eigen::MatrixXd > Eigy(omxMatrixDataColumnMajor(oge->y->dataMat), oge->y->dataMat->cols, 1);
   SEXP b_ext, bcov_ext, RyXcolnames;
   oge->quadXinv = oge->quadXinv.selfadjointView<Eigen::Lower>();
-  Eigen::MatrixXd GREML_b = oge->quadXinv * oge->XtVinv * Eigy;
+  Eigen::MatrixXd GREML_b = oge->quadXinv * (oge->XtVinv * Eigy);
   
   {
   ScopedProtect p1(b_ext, Rf_allocMatrix(REALSXP, GREML_b.rows(), 1));
