@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2017 The OpenMx Project
+ *  Copyright 2007-2018 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,10 +21,8 @@ struct omxRFitFunction : omxFitFunction {
 
 	SEXP fitfun;
 	SEXP model;
-	PROTECT_INDEX modelIndex;
 	SEXP flatModel;
 	SEXP state;
-	PROTECT_INDEX stateIndex;
 
 	virtual void init();
 	virtual void compute(int ffcompute, FitContext *fc);
